@@ -3,12 +3,18 @@ import os
 import sys
 import inspect
 import traceback
+from types import FunctionType, MethodType
 
 from . import cmd_with_io
 from .parameters import Parameter
-from .exception import MetaflowException, MetaflowInternalError, \
-    MissingInMergeArtifactsException, UnhandledInMergeArtifactsException
+from .exception import (
+    MetaflowException,
+    MetaflowInternalError,
+    MissingInMergeArtifactsException,
+    UnhandledInMergeArtifactsException,
+)
 from .graph import FlowGraph
+from .unbounded_foreach import UnboundedForeachInput
 
 # For Python 3 compatibility
 try:
@@ -27,6 +33,18 @@ class InvalidNextException(MetaflowException):
         super(InvalidNextException, self).__init__(msg, line_no)
 
 
+class ParallelUBF(UnboundedForeachInput):
+    """
+    Unbounded-for-each placeholder for supporting parallel (multi-node) steps.
+    """
+
+    def __init__(self, num_parallel):
+        self.num_parallel = num_parallel
+
+    def __getitem__(self, item):
+        return item or 0  # item is None for the control task, but it is also split 0
+
+
 class FlowSpec(object):
     """
     Main class from which all Flows should inherit.
@@ -41,14 +59,22 @@ class FlowSpec(object):
     # Attributes that are not saved in the datastore when checkpointing.
     # Name starting with '__', methods, functions and Parameters do not need
     # to be listed.
-    _EPHEMERAL = {'_EPHEMERAL',
-                  '_datastore',
-                  '_cached_input',
-                  '_graph',
-                  '_flow_decorators',
-                  '_steps',
-                  'index',
-                  'input'}
+    _EPHEMERAL = {
+        "_EPHEMERAL",
+        "_NON_PARAMETERS",
+        "_datastore",
+        "_cached_input",
+        "_graph",
+        "_flow_decorators",
+        "_steps",
+        "index",
+        "input",
+    }
+    # When checking for parameters, we look at dir(self) but we want to exclude
+    # attributes that are definitely not parameters and may be expensive to
+    # compute (like anything related to the `foreach_stack`). We don't need to exclude
+    # names starting with `_` as those are already excluded from `_get_parameters`.
+    _NON_PARAMETERS = {"cmd", "foreach_stack", "index", "input", "script_name", "name"}
 
     _flow_decorators = {}
 
@@ -75,6 +101,7 @@ class FlowSpec(object):
             # we import cli here to make sure custom parameters in
             # args.py get fully evaluated before cli.py is imported.
             from . import cli
+
             cli.main(self)
 
     @property
@@ -88,13 +115,51 @@ class FlowSpec(object):
             A string containing the name of the script
         """
         fname = inspect.getfile(self.__class__)
-        if fname.endswith('.pyc'):
+        if fname.endswith(".pyc"):
             fname = fname[:-1]
         return os.path.basename(fname)
 
+    def _set_constants(self, kwargs):
+        # Persist values for parameters and other constants (class level variables)
+        # only once. This method is called before persist_constants is called to
+        # persist all values set using setattr
+        seen = set()
+        for var, param in self._get_parameters():
+            norm = param.name.lower()
+            if norm in seen:
+                raise MetaflowException(
+                    "Parameter *%s* is specified twice. "
+                    "Note that parameter names are "
+                    "case-insensitive." % param.name
+                )
+            seen.add(norm)
+        seen.clear()
+        self._success = True
+
+        for var, param in self._get_parameters():
+            seen.add(var)
+            val = kwargs[param.name.replace("-", "_").lower()]
+            # Support for delayed evaluation of parameters. This is used for
+            # includefile in particular
+            if callable(val):
+                val = val()
+            val = val.split(param.separator) if val and param.separator else val
+            setattr(self, var, val)
+
+        # Do the same for class variables which will be forced constant as modifications
+        # to them don't propagate well since we create a new process for each step and
+        # re-read the flow file
+        for var in dir(self.__class__):
+            if var[0] == "_" or var in self._NON_PARAMETERS or var in seen:
+                continue
+            val = getattr(self.__class__, var)
+            if isinstance(val, (MethodType, FunctionType, property, type)):
+                continue
+            setattr(self, var, val)
+
     def _get_parameters(self):
         for var in dir(self):
-            if var[0] == '_':
+            if var[0] == "_" or var in self._NON_PARAMETERS:
                 continue
             try:
                 val = getattr(self, var)
@@ -125,13 +190,10 @@ class FlowSpec(object):
             setattr(self, name, x)
             return x
         else:
-            raise AttributeError("Flow %s has no attribute '%s'" %
-                                 (self.name, name))
+            raise AttributeError("Flow %s has no attribute '%s'" % (self.name, name))
 
     def cmd(self, cmdline, input={}, output=[]):
-        return cmd_with_io.cmd(cmdline,
-                               input=input,
-                               output=output)
+        return cmd_with_io.cmd(cmdline, input=input, output=output)
 
     @property
     def index(self):
@@ -218,11 +280,12 @@ class FlowSpec(object):
         List[Tuple[int, int, object]]
             An array describing the current stack of foreach steps
         """
-        return [(frame.index, frame.num_splits, self._find_input(stack_index=i))
-                for i, frame in enumerate(self._foreach_stack)]
+        return [
+            (frame.index, frame.num_splits, self._find_input(stack_index=i))
+            for i, frame in enumerate(self._foreach_stack)
+        ]
 
     def _find_input(self, stack_index=None):
-
         if stack_index is None:
             stack_index = len(self._foreach_stack) - 1
 
@@ -249,9 +312,9 @@ class FlowSpec(object):
                     self._cached_input[stack_index] = var[frame.index]
                 except TypeError:
                     # __getitem__ not supported, fall back to an iterator
-                    self._cached_input[stack_index] = next(islice(var,
-                                                                  frame.index,
-                                                                  frame.index + 1))
+                    self._cached_input[stack_index] = next(
+                        islice(var, frame.index, frame.index + 1)
+                    )
             return self._cached_input[stack_index]
 
     def merge_artifacts(self, inputs, exclude=[], include=[]):
@@ -306,9 +369,11 @@ class FlowSpec(object):
             be found
         """
         node = self._graph[self._current_step]
-        if node.type != 'join':
-            msg = "merge_artifacts can only be called in a join and step *{step}* "\
-                  "is not a join".format(step=self._current_step)
+        if node.type != "join":
+            msg = (
+                "merge_artifacts can only be called in a join and step *{step}* "
+                "is not a join".format(step=self._current_step)
+            )
             raise MetaflowException(msg)
         if len(exclude) > 0 and len(include) > 0:
             msg = "`exclude` and `include` are mutually exclusive in merge_artifacts"
@@ -319,11 +384,17 @@ class FlowSpec(object):
         for inp in inputs:
             # available_vars is the list of variables from inp that should be considered
             if include:
-                available_vars = ((var, sha) for var, sha in inp._datastore.items()
-                                  if (var in include) and (not hasattr(self, var)))
+                available_vars = (
+                    (var, sha)
+                    for var, sha in inp._datastore.items()
+                    if (var in include) and (not hasattr(self, var))
+                )
             else:
-                available_vars = ((var, sha) for var, sha in inp._datastore.items()
-                                if (var not in exclude) and (not hasattr(self, var)))
+                available_vars = (
+                    (var, sha)
+                    for var, sha in inp._datastore.items()
+                    if (var not in exclude) and (not hasattr(self, var))
+                )
             for var, sha in available_vars:
                 _, previous_sha = to_merge.setdefault(var, (inp, sha))
                 if previous_sha != sha:
@@ -336,24 +407,49 @@ class FlowSpec(object):
                 missing.append(v)
         if unresolved:
             # We have unresolved conflicts so we do not set anything and error out
-            msg = "Step *{step}* cannot merge the following artifacts due to them "\
-                  "having conflicting values:\n[{artifacts}].\nTo remedy this issue, "\
-                  "be sure to explictly set those artifacts (using "\
-                  "self.<artifact_name> = ...) prior to calling merge_artifacts."\
-                  .format(step=self._current_step, artifacts=', '.join(unresolved))
+            msg = (
+                "Step *{step}* cannot merge the following artifacts due to them "
+                "having conflicting values:\n[{artifacts}].\nTo remedy this issue, "
+                "be sure to explicitly set those artifacts (using "
+                "self.<artifact_name> = ...) prior to calling merge_artifacts.".format(
+                    step=self._current_step, artifacts=", ".join(unresolved)
+                )
+            )
             raise UnhandledInMergeArtifactsException(msg, unresolved)
         if missing:
-            msg = "Step *{step}* specifies that [{include}] should be merged but "\
-                  "[{missing}] are not present.\nTo remedy this issue, make sure "\
-                  "that the values specified in only come from at least one branch"\
-                  .format(
-                      step=self._current_step,
-                      include=', '.join(include),
-                      missing=', '.join(missing))
+            msg = (
+                "Step *{step}* specifies that [{include}] should be merged but "
+                "[{missing}] are not present.\nTo remedy this issue, make sure "
+                "that the values specified in only come from at least one branch".format(
+                    step=self._current_step,
+                    include=", ".join(include),
+                    missing=", ".join(missing),
+                )
+            )
             raise MissingInMergeArtifactsException(msg, missing)
-        # If things are resolved, we go and fetch from the datastore and set here
+        # If things are resolved, we pass down the variables from the input datastores
         for var, (inp, _) in to_merge.items():
-            setattr(self, var, getattr(inp, var))
+            self._datastore.passdown_partial(inp._datastore, [var])
+
+    def _validate_ubf_step(self, step_name):
+        join_list = self._graph[step_name].out_funcs
+        if len(join_list) != 1:
+            msg = (
+                "UnboundedForeach is supported only over a single node, "
+                "not an arbitrary DAG. Specify a single `join` node"
+                " instead of multiple:{join_list}.".format(join_list=join_list)
+            )
+            raise InvalidNextException(msg)
+        join_step = join_list[0]
+        join_node = self._graph[join_step]
+        join_type = join_node.type
+
+        if join_type != "join":
+            msg = (
+                "UnboundedForeach found for:{node} -> {join}."
+                " The join type isn't valid.".format(node=step_name, join=join_step)
+            )
+            raise InvalidNextException(msg)
 
     def next(self, *dsts, **kwargs):
         """
@@ -382,18 +478,23 @@ class FlowSpec(object):
 
         step = self._current_step
 
-        foreach = kwargs.pop('foreach', None)
-        condition = kwargs.pop('condition', None)
+        foreach = kwargs.pop("foreach", None)
+        num_parallel = kwargs.pop("num_parallel", None)
+        condition = kwargs.pop("condition", None)
         if kwargs:
             kw = next(iter(kwargs))
-            msg = "Step *{step}* passes an unknown keyword argument "\
-                  "'{invalid}' to self.next().".format(step=step, invalid=kw)
+            msg = (
+                "Step *{step}* passes an unknown keyword argument "
+                "'{invalid}' to self.next().".format(step=step, invalid=kw)
+            )
             raise InvalidNextException(msg)
 
         # check: next() is called only once
         if self._transition is not None:
-            msg = "Multiple self.next() calls detected in step *{step}*. "\
-                  "Call self.next() only once.".format(step=step)
+            msg = (
+                "Multiple self.next() calls detected in step *{step}*. "
+                "Call self.next() only once.".format(step=step)
+            )
             raise InvalidNextException(msg)
 
         # check: all destinations are methods of this object
@@ -402,111 +503,148 @@ class FlowSpec(object):
             try:
                 name = dst.__func__.__name__
             except:
-                msg = "In step *{step}* the {arg}. argument in self.next() is "\
-                      "not a function. Make sure all arguments in self.next() "\
-                      "are methods of the Flow class."\
-                      .format(step=step, arg=i + 1)
+                msg = (
+                    "In step *{step}* the {arg}. argument in self.next() is "
+                    "not a function. Make sure all arguments in self.next() "
+                    "are methods of the Flow class.".format(step=step, arg=i + 1)
+                )
                 raise InvalidNextException(msg)
             if not hasattr(self, name):
-                msg = "Step *{step}* specifies a self.next() transition to an "\
-                      "unknown step, *{name}*.".format(step=step,
-                                                       name=name)
+                msg = (
+                    "Step *{step}* specifies a self.next() transition to an "
+                    "unknown step, *{name}*.".format(step=step, name=name)
+                )
                 raise InvalidNextException(msg)
             funcs.append(name)
 
+        if num_parallel is not None and num_parallel >= 1:
+            if len(dsts) > 1:
+                raise InvalidNextException(
+                    "Only one destination allowed when num_parallel used in self.next()"
+                )
+            foreach = "_parallel_ubf_iter"
+            self._parallel_ubf_iter = ParallelUBF(num_parallel)
+
         # check: foreach and condition are mutually exclusive
         if not (foreach is None or condition is None):
-            msg = "Step *{step}* has an invalid self.next() transition. "\
-                  "Specify either 'foreach' or 'condition', not both."\
-                  .format(step=step)
+            msg = (
+                "Step *{step}* has an invalid self.next() transition. "
+                "Specify either 'foreach' or 'condition', not both.".format(step=step)
+            )
             raise InvalidNextException(msg)
 
         # check: foreach is valid
         if foreach:
             if not isinstance(foreach, basestring):
-                msg = "Step *{step}* has an invalid self.next() transition. "\
-                      "The argument to 'foreach' must be a string."\
-                      .format(step=step)
+                msg = (
+                    "Step *{step}* has an invalid self.next() transition. "
+                    "The argument to 'foreach' must be a string.".format(step=step)
+                )
                 raise InvalidNextException(msg)
 
             if len(dsts) != 1:
-                msg = "Step *{step}* has an invalid self.next() transition. "\
-                      "Specify exactly one target for 'foreach'."\
-                      .format(step=step)
+                msg = (
+                    "Step *{step}* has an invalid self.next() transition. "
+                    "Specify exactly one target for 'foreach'.".format(step=step)
+                )
                 raise InvalidNextException(msg)
 
             try:
                 foreach_iter = getattr(self, foreach)
             except:
-                msg = "Foreach variable *self.{var}* in step *{step}* "\
-                      "does not exist. Check your variable."\
-                      .format(step=step, var=foreach)
+                msg = (
+                    "Foreach variable *self.{var}* in step *{step}* "
+                    "does not exist. Check your variable.".format(
+                        step=step, var=foreach
+                    )
+                )
                 raise InvalidNextException(msg)
 
-            try:
-                self._foreach_num_splits = sum(1 for _ in foreach_iter)
-            except TypeError:
-                msg = "Foreach variable *self.{var}* in step *{step}* "\
-                      "is not iterable. Check your variable."\
-                      .format(step=step, var=foreach)
-                raise InvalidNextException(msg)
+            if issubclass(type(foreach_iter), UnboundedForeachInput):
+                self._unbounded_foreach = True
+                self._foreach_num_splits = None
+                self._validate_ubf_step(funcs[0])
+            else:
+                try:
+                    self._foreach_num_splits = sum(1 for _ in foreach_iter)
+                except TypeError:
+                    msg = (
+                        "Foreach variable *self.{var}* in step *{step}* "
+                        "is not iterable. Check your variable.".format(
+                            step=step, var=foreach
+                        )
+                    )
+                    raise InvalidNextException(msg)
 
-            if self._foreach_num_splits == 0:
-                msg = "Foreach iterator over *{var}* in step *{step}* "\
-                      "produced zero splits. Check your variable."\
-                      .format(step=step, var=foreach)
-                raise InvalidNextException(msg)
+                if self._foreach_num_splits == 0:
+                    msg = (
+                        "Foreach iterator over *{var}* in step *{step}* "
+                        "produced zero splits. Check your variable.".format(
+                            step=step, var=foreach
+                        )
+                    )
+                    raise InvalidNextException(msg)
 
             self._foreach_var = foreach
 
         # check: condition is valid
         if condition:
             if not isinstance(condition, basestring):
-                msg = "Step *{step}* has an invalid self.next() transition. "\
-                      "The argument to 'condition' must be a string."\
-                      .format(step=step)
+                msg = (
+                    "Step *{step}* has an invalid self.next() transition. "
+                    "The argument to 'condition' must be a string.".format(step=step)
+                )
                 raise InvalidNextException(msg)
             if len(dsts) != 2:
-                msg = "Step *{step}* has an invalid self.next() transition. "\
-                      "Specify two targets for 'condition': The first target "\
-                      "is used if the condition evaluates to true, the second "\
-                      "otherwise.".format(step=step)
+                msg = (
+                    "Step *{step}* has an invalid self.next() transition. "
+                    "Specify two targets for 'condition': The first target "
+                    "is used if the condition evaluates to true, the second "
+                    "otherwise.".format(step=step)
+                )
                 raise InvalidNextException(msg)
 
         # check: non-keyword transitions are valid
         if foreach is None and condition is None:
             if len(dsts) < 1:
-                msg = "Step *{step}* has an invalid self.next() transition. "\
-                      "Specify at least one step function as an argument in "\
-                      "self.next().".format(step=step)
+                msg = (
+                    "Step *{step}* has an invalid self.next() transition. "
+                    "Specify at least one step function as an argument in "
+                    "self.next().".format(step=step)
+                )
                 raise InvalidNextException(msg)
 
         self._transition = (funcs, foreach, condition)
 
     def __str__(self):
-        step_name = getattr(self, '_current_step', None)
+        step_name = getattr(self, "_current_step", None)
         if step_name:
-            index = ','.join(str(idx) for idx, _, _ in self.foreach_stack())
+            index = ",".join(str(idx) for idx, _, _ in self.foreach_stack())
             if index:
                 inp = self.input
                 if inp is None:
-                    return '<flow %s step %s[%s]>' %\
-                           (self.name, step_name, index)
+                    return "<flow %s step %s[%s]>" % (self.name, step_name, index)
                 else:
                     inp = str(inp)
                     if len(inp) > 20:
-                        inp = inp[:20] + '...'
-                    return '<flow %s step %s[%s] (input: %s)>' %\
-                        (self.name, step_name, index, inp)
+                        inp = inp[:20] + "..."
+                    return "<flow %s step %s[%s] (input: %s)>" % (
+                        self.name,
+                        step_name,
+                        index,
+                        inp,
+                    )
             else:
-                return '<flow %s step %s>' % (self.name, step_name)
+                return "<flow %s step %s>" % (self.name, step_name)
         else:
-            return '<flow %s>' % self.name
+            return "<flow %s>" % self.name
 
     def __getstate__(self):
-        raise MetaflowException("Flows can't be serialized. Maybe you tried "
-                                "to assign *self* or one of the *inputs* "
-                                "to an attribute? Instead of serializing the "
-                                "whole flow, you should choose specific "
-                                "attributes, e.g. *input.some_var*, to be "
-                                "stored.")
+        raise MetaflowException(
+            "Flows can't be serialized. Maybe you tried "
+            "to assign *self* or one of the *inputs* "
+            "to an attribute? Instead of serializing the "
+            "whole flow, you should choose specific "
+            "attributes, e.g. *input.some_var*, to be "
+            "stored."
+        )
