@@ -12,11 +12,11 @@ version_cache = None
 
 
 class InvalidEnvironmentException(MetaflowException):
-    headline = 'Incompatible environment'
+    headline = "Incompatible environment"
 
 
 class MetaflowEnvironment(object):
-    TYPE = 'local'
+    TYPE = "local"
 
     def __init__(self, flow):
         pass
@@ -50,7 +50,7 @@ class MetaflowEnvironment(object):
     def add_to_package(self):
         """
         A list of tuples (file, arcname) to add to the job package.
-        `arcname` is an alterative name for the file in the job package.
+        `arcname` is an alternative name for the file in the job package.
         """
         return []
 
@@ -79,23 +79,26 @@ class MetaflowEnvironment(object):
         return "Local environment"
 
     def get_package_commands(self, code_package_url):
-        cmds = [BASH_MFLOG,
-                "%s -m pip install awscli click requests boto3 aws-requests-auth -qqq" 
-                    % self._python(),
-                "mkdir metaflow",
-                "cd metaflow",
-                "mkdir .metaflow", # mute local datastore creation log
-                "i=0; while [ $i -le 5 ]; do "
-                    "%s -m awscli s3 cp %s job.tar >/dev/null && \
-                        break; "
-                    "sleep 10; i=$((i+1)); "
-                "done" % (self._python(), code_package_url),
-                "if [ $i -gt 5 ]; then "
-                    "exit 1; "
-                "fi",
-                "tar xf job.tar",
-                "mflog \'Task is starting.\'",
-                ]
+        cmds = [
+            BASH_MFLOG,
+            "mflog 'Setting up task environment.'",
+            "%s -m pip install awscli click requests boto3 -qqq" % self._python(),
+            "mkdir metaflow",
+            "cd metaflow",
+            "mkdir .metaflow",  # mute local datastore creation log
+            "i=0; while [ $i -le 5 ]; do "
+            "mflog 'Downloading code package...'; "
+            "%s -m awscli s3 cp %s job.tar >/dev/null && \
+                        mflog 'Code package downloaded.' && break; "
+            "sleep 10; i=$((i+1)); "
+            "done" % (self._python(), code_package_url),
+            "if [ $i -gt 5 ]; then "
+            "mflog 'Failed to download code package from %s "
+            "after 6 tries. Exiting...' && exit 1; "
+            "fi" % code_package_url,
+            "TAR_OPTIONS='--warning=no-timestamp' tar xf job.tar",
+            "mflog 'Task is starting.'",
+        ]
         return cmds
 
     def get_environment_info(self):
@@ -106,21 +109,23 @@ class MetaflowEnvironment(object):
         # note that this dict goes into the code package
         # so variables here should be relatively stable (no
         # timestamps) so the hash won't change all the time
-        env = {'platform': platform.system(),
-               'username': get_username(),
-               'production_token': os.environ.get('METAFLOW_PRODUCTION_TOKEN'),
-               'runtime': os.environ.get('METAFLOW_RUNTIME_NAME', 'dev'),
-               'app': os.environ.get('APP'),
-               'environment_type': self.TYPE,
-               'use_r': R.use_r(),
-               'python_version': sys.version,
-               'python_version_code': '%d.%d.%d' % sys.version_info[:3],
-               'metaflow_version': version_cache,
-               'script': os.path.basename(os.path.abspath(sys.argv[0]))}
+        env = {
+            "platform": platform.system(),
+            "username": get_username(),
+            "production_token": os.environ.get("METAFLOW_PRODUCTION_TOKEN"),
+            "runtime": os.environ.get("METAFLOW_RUNTIME_NAME", "dev"),
+            "app": os.environ.get("APP"),
+            "environment_type": self.TYPE,
+            "use_r": R.use_r(),
+            "python_version": sys.version,
+            "python_version_code": "%d.%d.%d" % sys.version_info[:3],
+            "metaflow_version": version_cache,
+            "script": os.path.basename(os.path.abspath(sys.argv[0])),
+        }
         if R.use_r():
-            env['metaflow_r_version'] = R.metaflow_r_version()
-            env['r_version'] = R.r_version()
-            env['r_version_code'] = R.r_version_code()
+            env["metaflow_r_version"] = R.metaflow_r_version()
+            env["r_version"] = R.r_version()
+            env["r_version_code"] = R.r_version_code()
         return env
 
     def executable(self, step_name):
@@ -130,4 +135,4 @@ class MetaflowEnvironment(object):
         if R.use_r():
             return "python3"
         else:
-            return "python"
\ No newline at end of file
+            return "python"
