@@ -1,5 +1,6 @@
 # File modified under the Apache Licence 2.0 spec. Modified by Aero Technologies.
 # Altered how profile is set based on local/remote execution
+from .auth import AWSRequestsAuth
 import logging
 import os
 import urllib.parse
@@ -118,8 +119,23 @@ def get_aws_client(module, with_error=False, params={}):
     return AWS_SESSION.client(module, **params)
 
 
-from aws_requests_auth.aws_auth import AWSRequestsAuth
-from aws_requests_auth.boto_utils import get_credentials
+class Boto3ClientProvider(object):
+    name = "boto3"
+
+    @staticmethod
+    def get_client(module, with_error=False, params={}):
+        from metaflow.exception import MetaflowException
+
+        try:
+            import boto3
+            from botocore.exceptions import ClientError
+        except (NameError, ImportError):
+            raise MetaflowException(
+                "Could not import module 'boto3'. Install boto3 first.")
+
+        if with_error:
+            return AWS_SESSION.client(module, **params), ClientError
+        return AWS_SESSION.client(module, **params)
 
 class AeroAWSRequestsAuth(AWSRequestsAuth):
 
@@ -131,12 +147,18 @@ class AeroAWSRequestsAuth(AWSRequestsAuth):
     def get_aws_request_headers_handler(self, r):
         # provide credentials explicitly during each __call__, to take advantage
         # of botocore's underlying logic to refresh expired credentials
-        credentials = get_credentials(self._refreshable_credentials)
+        frozen_credentials = self._refreshable_credentials.get_frozen_credentials()
+        credentials = {
+            'aws_access_key': frozen_credentials.access_key,
+            'aws_secret_access_key': frozen_credentials.secret_key,
+            'aws_token': frozen_credentials.token,
+        }
+
         return self.get_aws_request_headers(r, **credentials)
 
 def get_auth_object(
     url: str,
-    region: str = 'eu-west-1', 
+    region: str = 'eu-west-1',
     service: str = 'execute-api'
 ):
     split_url = url.split("/")
@@ -146,15 +168,15 @@ def get_auth_object(
     )
 
 def get_signed_url(
-        url: str,
-        method: str,
-        dt: datetime = None,
-        region: str = 'eu-west-1', 
-        service: str = 'execute-api', 
-        protocol: str = 'https://', 
-        query_string_params: str = None,
-        payload: str = ""
-    ):
+    url: str,
+    method: str,
+    dt: datetime = None,
+    region: str = 'eu-west-1',
+    service: str = 'execute-api',
+    protocol: str = 'https://',
+    query_string_params: str = None,
+    payload: str = ""
+):
 
     credentials = _get_base_session_credentials()
     # eg: "https://api.aeroplatform.co.uk/service-api/flows/DailyScheduleTest2/runs/15/steps/end/tasks"
