@@ -10,7 +10,6 @@ from metaflow import current, decorators, parameters, JSONType
 from metaflow.metaflow_config import METADATA_SERVICE_AUTH_KEY
 
 from metaflow.exception import MetaflowException, MetaflowInternalError
-from metaflow.datastore.datastore import TransformableObject
 from metaflow.package import MetaflowPackage
 from metaflow.plugins import BatchDecorator
 from metaflow.util import get_username, to_bytes, to_unicode
@@ -25,12 +24,15 @@ EXIT_KEYWORD = "Exit"
 class IncorrectProductionToken(MetaflowException):
     headline = "Incorrect production token"
 
+
 class IncorrectMetadataServiceVersion(MetaflowException):
     headline = "Incorrect version for metaflow service"
 
+
 class StepFunctionsStateMachineNameTooLong(MetaflowException):
     headline = "Aero Scheduler state machine name too long"
 
+
 @click.group()
 def cli():
     pass
@@ -45,10 +47,10 @@ def cli():
 def schedule(obj, name=None):
     obj.check(obj.graph, obj.flow, obj.environment, pylint=obj.pylint)
     obj.state_machine_name, obj.token_prefix, obj.is_project = \
-                                      resolve_state_machine_name(obj, name)
+        resolve_state_machine_name(obj, name)
 
 @schedule.command(help="Deploy a new version of this workflow to "
-                    "the Aero scheduler.")
+                  "the Aero scheduler.")
 @click.option('--authorize',
               default=None,
               help="Authorize using this production token. You need this "
@@ -111,22 +113,26 @@ def create(obj,
 
     check_metadata_service_version(obj)
 
-    token = resolve_token(obj.state_machine_name,
-                          obj.token_prefix,
-                          obj,
-                          authorize,
-                          given_token,
-                          generate_new_token,
-                          obj.is_project)
+    token = resolve_token(
+        obj.state_machine_name,
+        obj.token_prefix,
+        obj,
+        authorize,
+        given_token,
+        generate_new_token,
+        obj.is_project,
+    )
 
-    flow = make_flow(obj,
-                     token,
-                     obj.state_machine_name,
-                     tags,
-                     user_namespace,
-                     max_workers,
-                     workflow_timeout,
-                     obj.is_project)
+    flow = make_flow(
+        obj,
+        token,
+        obj.state_machine_name,
+        tags,
+        user_namespace,
+        max_workers,
+        workflow_timeout,
+        obj.is_project,
+    )
 
     if only_json:
         obj.echo_always(flow.to_json(), err=False, no_bold=True)
@@ -135,8 +141,8 @@ def create(obj,
         obj.echo("State Machine *{state_machine}* "
                  "for flow *{name}* pushed to "
                  "Aero Scheduler successfully.\n"
-                    .format(state_machine=display_state_machine_name(obj.state_machine_name),
-                            name=current.flow_name), 
+                 .format(state_machine=display_state_machine_name(obj.state_machine_name),
+                         name=current.flow_name),
                  bold=True)
         if obj._is_state_machine_name_hashed:
             obj.echo("Note that the flow was deployed with a truncated name "
@@ -145,13 +151,15 @@ def create(obj,
         flow.schedule()
         obj.echo("What will trigger execution of the workflow:", bold=True)
         obj.echo(flow.trigger_explanation(), indent=True)
-        
+
+# Aero Technology: Updating display name to include username
 def display_state_machine_name(name):
     try:
         split = name.split("_")
         return "_".join(split[1:])
     except:
         return name
+# Aero Technology
 
 def check_metadata_service_version(obj):
     return
@@ -164,29 +172,29 @@ def resolve_state_machine_name(obj, name):
     obj._is_state_machine_name_hashed = False
     if project:
         if name:
-            raise MetaflowException("--name is not supported for @projects. "
-                                    "Use --branch instead.")
+            raise MetaflowException(
+                "--name is not supported for @projects. " "Use --branch instead."
+            )
         state_machine_name = attach_prefix(current.project_flow_name)
         project_branch = to_bytes('.'.join((project, current.branch_name)))
         token_prefix = 'mfprj-%s' % to_unicode(
-                                        base64.b32encode(
-                                            sha1(project_branch).digest()))[:16]
+            base64.b32encode(
+                sha1(project_branch).digest()))[:16]
         is_project = True
         # AWS Step Functions has a limit of 80 chars for state machine names.
         # We truncate the state machine name if the computed name is greater
         # than 60 chars and append a hashed suffix to ensure uniqueness.
         if len(state_machine_name) > 60:
             name_hash = to_unicode(
-                            base64.b32encode(
-                                sha1(to_bytes(state_machine_name)) \
-                                  .digest()))[:16].lower()
+                        base64.b32encode(
+                            sha1(to_bytes(state_machine_name)) \
+                    .digest()))[:16].lower()
             state_machine_name =\
                 '%s-%s' % (state_machine_name[:60], name_hash)
             obj._is_state_machine_name_hashed = True
     else:
         if name and VALID_NAME.search(name):
-            raise MetaflowException(
-                "Name '%s' contains invalid characters." % name)
+            raise MetaflowException("Name '%s' contains invalid characters." % name)
 
         state_machine_name = attach_prefix(name if name else current.flow_name)
         token_prefix = state_machine_name
@@ -210,49 +218,46 @@ def make_flow(obj,
               max_workers,
               workflow_timeout,
               is_project):
-    datastore = obj.datastore(obj.flow.name,
-                              mode='w',
-                              metadata=obj.metadata,
-                              event_logger=obj.event_logger,
-                              monitor=obj.monitor)
-    if datastore.TYPE != 's3':
+
+    if obj.flow_datastore.TYPE != 's3':
         raise MetaflowException("Aero scheduler requires --datastore=s3.")
 
     # Attach AWS Batch decorator to the flow
     decorators._attach_decorators(obj.flow, [BatchDecorator.name])
     decorators._init_step_decorators(
-            obj.flow, obj.graph, obj.environment, obj.datastore, obj.logger)
+        obj.flow, obj.graph, obj.environment, obj.flow_datastore, obj.logger)
 
     obj.package = MetaflowPackage(
-        obj.flow, obj.environment, obj.echo, obj.package_suffixes)
-    package_url = datastore.save_data(
-        obj.package.sha, TransformableObject(obj.package.blob))
+        obj.flow, obj.environment, obj.echo, obj.package_suffixes
+    )
+    package_url, package_sha = obj.flow_datastore.save_data(
+        [obj.package.blob], len_hint=1
+    )[0]
 
-    return StepFunctions(name,
-                         obj.graph,
-                         obj.flow,
-                         obj.package,
-                         package_url,
-                         token,
-                         obj.metadata,
-                         obj.datastore,
-                         obj.environment,
-                         obj.event_logger,
-                         obj.monitor,
-                         tags=tags,
-                         namespace=namespace,
-                         max_workers=max_workers,
-                         username=get_username(),
-                         workflow_timeout=workflow_timeout,
-                         is_project=is_project)
+    return StepFunctions(
+        name,
+        obj.graph,
+        obj.flow,
+        package_sha,
+        package_url,
+        token,
+        obj.metadata,
+        obj.flow_datastore,
+        obj.environment,
+        obj.event_logger,
+        obj.monitor,
+        tags=tags,
+        namespace=namespace,
+        max_workers=max_workers,
+        username=get_username(),
+        workflow_timeout=workflow_timeout,
+        is_project=is_project,
+    )
 
-def resolve_token(name,
-                  token_prefix,
-                  obj,
-                  authorize,
-                  given_token,
-                  generate_new_token,
-                  is_project):
+
+def resolve_token(
+    name, token_prefix, obj, authorize, given_token, generate_new_token, is_project
+):
 
     # 1) retrieve the previous deployment, if one exists
     workflow = StepFunctions.get_existing_deployment(name)
@@ -267,7 +272,7 @@ def resolve_token(name,
     if prev_token is not None:
         if authorize is None:
             authorize = load_token(token_prefix)
-        elif authorize.startswith('production:'):
+        elif authorize.startswith("production:"):
             authorize = authorize[11:]
 
         # we allow the user who deployed the previous version to re-deploy,
@@ -280,20 +285,23 @@ def resolve_token(name,
         if is_project:
             # we rely on a known prefix for @project tokens, so we can't
             # allow the user to specify a custom token with an arbitrary prefix
-            raise MetaflowException("--new-token is not supported for "
-                                    "@projects. Use --generate-new-token to "
-                                    "create a new token.")
-        if given_token.startswith('production:'):
+            raise MetaflowException(
+                "--new-token is not supported for "
+                "@projects. Use --generate-new-token to "
+                "create a new token."
+            )
+        if given_token.startswith("production:"):
             given_token = given_token[11:]
         token = given_token
-        obj.echo('')
+        obj.echo("")
         obj.echo("Using the given token, *%s*." % token)
     elif prev_token is None or generate_new_token:
         token = new_token(token_prefix, prev_token)
         if token is None:
             if prev_token is None:
-                raise MetaflowInternalError("We could not generate a new "
-                                            "token. This is unexpected. ")
+                raise MetaflowInternalError(
+                    "We could not generate a new " "token. This is unexpected. "
+                )
             else:
                 raise MetaflowException("--generate-new-token option is not "
                                         "supported after using --new-token. "
@@ -305,7 +313,7 @@ def resolve_token(name,
 
     obj.echo('')
     obj.echo("To analyze results from a production flow, use")
-    obj.echo('    namespace("production")' , fg='green')
+    obj.echo('    namespace("production")', fg='green')
     obj.echo("This will show results for any scheduled flows. ")
     obj.echo('')
     store_token(token_prefix, token)
@@ -315,7 +323,7 @@ def resolve_token(name,
 @parameters.add_custom_parameters(deploy_mode=False)
 @schedule.command(help="Trigger the workflow on the Aero scheduler.")
 @click.pass_obj
-def trigger(obj, **kwargs):
+def trigger(obj, run_id_file=None, **kwargs):
     def _convert_value(param):
         # Swap `-` with `_` in parameter name to match click's behavior
         val = kwargs.get(param.name.replace('-', '_').lower())
@@ -324,35 +332,35 @@ def trigger(obj, **kwargs):
 
     params = {param.name: _convert_value(param)
               for _, param in obj.flow._get_parameters()
-                if kwargs.get(param.name.replace('-', '_').lower()) is not None}
+              if kwargs.get(param.name.replace('-', '_').lower()) is not None}
 
     response = StepFunctions.trigger(obj.state_machine_name, params)
 
     id = response['executionArn'].split(':')[-1]
     obj.echo("Workflow *{name}* triggered on Aero Scheduler "
-        "(run-id *sfn-{id}*)."
-            .format(name=display_state_machine_name(obj.state_machine_name), id=id), bold=True)
+             "(run-id *sfn-{id}*)."
+             .format(name=display_state_machine_name(obj.state_machine_name), id=id), bold=True)
 
 
 @schedule.command(
     help="List all runs of the workflow on Aero scheduler.")
 @click.option("--running", default=False, is_flag=True,
-    help="List all runs of the workflow in RUNNING state on "
-        "the Aero scheduler.")
+              help="List all runs of the workflow in RUNNING state on "
+              "the Aero scheduler.")
 @click.option("--succeeded", default=False, is_flag=True,
-    help="List all runs of the workflow in SUCCEEDED state on "
-        "the Aero scheduler.")
+              help="List all runs of the workflow in SUCCEEDED state on "
+              "the Aero scheduler.")
 @click.option("--failed", default=False, is_flag=True,
-    help="List all runs of the workflow in FAILED state on "
-        "the Aero scheduler.")
+              help="List all runs of the workflow in FAILED state on "
+              "the Aero scheduler.")
 @click.option("--timed-out", default=False, is_flag=True,
-    help="List all runs of the workflow in TIMED_OUT state on "
-        "the Aero scheduler.")
+              help="List all runs of the workflow in TIMED_OUT state on "
+              "the Aero scheduler.")
 @click.option("--aborted", default=False, is_flag=True,
-    help="List all runs of the workflow in ABORTED state on "
-        "the Aero scheduler.")
+              help="List all runs of the workflow in ABORTED state on "
+              "the Aero scheduler.")
 @click.pass_obj
-def list_runs(obj, 
+def list_runs(obj,
               running=False,
               succeeded=False,
               failed=False,
@@ -360,28 +368,28 @@ def list_runs(obj,
               aborted=False):
     states = []
     if running:
-        states.append('RUNNING')
+        states.append("RUNNING")
     if succeeded:
-        states.append('SUCCEEDED')
+        states.append("SUCCEEDED")
     if failed:
-        states.append('FAILED')
+        states.append("FAILED")
     if timed_out:
-        states.append('TIMED_OUT')
+        states.append("TIMED_OUT")
     if aborted:
-        states.append('ABORTED')
+        states.append("ABORTED")
     executions = StepFunctions.list(obj.state_machine_name, states)
     found = False
     for execution in executions:
         found = True
-        if execution.get('stopDate'):
+        if execution.get("stopDate"):
             obj.echo(
                 "*sfn-{id}* "
                 "startedAt:'{startDate}' "
                 "stoppedAt:'{stopDate}' "
                 "*{status}*".format(
-                    id=execution['name'], 
-                    status=execution['status'], 
-                    startDate=execution['startDate'].replace(microsecond=0), 
+                    id=execution['name'],
+                    status=execution['status'],
+                    startDate=execution['startDate'].replace(microsecond=0),
                     stopDate=execution['stopDate'].replace(microsecond=0),
                 )
             )
@@ -390,38 +398,68 @@ def list_runs(obj,
                 "*sfn-{id}* "
                 "startedAt:'{startDate}' "
                 "*{status}*".format(
-                    id=execution['name'], 
-                    status=execution['status'], 
+                    id=execution['name'],
+                    status=execution['status'],
                     startDate=execution['startDate'].replace(microsecond=0)
                 )
             )
     if not found:
         if len(states) > 0:
-            status = ''
+            status = ""
             for idx, state in enumerate(states):
                 if idx == 0:
                     pass
                 elif idx == len(states) - 1:
-                    status += ' and '
+                    status += " and "
                 else:
                     status += ', '
                 status += '*%s*' % state
             obj.echo('No %s executions for *%s* found on Aero Scheduler.'
-                % (status, display_state_machine_name(obj.state_machine_name)))
+                     % (status, display_state_machine_name(obj.state_machine_name)))
         else:
             obj.echo('No executions for *%s* found on Aero Scheduler.' \
-                % display_state_machine_name(obj.state_machine_name))
+                     % display_state_machine_name(obj.state_machine_name))
+
+
+@schedule.command(
+    help="Removes a scheduled flow")
+@click.option("--flow-id", default=None,
+              help="Flow to remove")
+@click.pass_obj
+def remove(obj, flow_id=None):
+
+    from ..aws_client import get_aws_client
+
+    # Get AWS Client
+    sfn = get_aws_client('stepfunctions')
+
+    functions = sfn.list_state_machines()['stateMachines']
+
+    function_to_delete = find_parameter(
+        functions, obj.state_machine_name, 'name')
+
+    if function_to_delete is None:
+        obj.echo('Not able to find %s on Aero Scheduler.'
+                 % (display_state_machine_name(obj.state_machine_name)), fg='red')
+        return
+
+    resp = sfn.delete_state_machine(
+        stateMachineArn=function_to_delete['stateMachineArn']
+    )
+
+    obj.echo('Removed %s from Aero Scheduler.'
+             % (display_state_machine_name(obj.state_machine_name)), fg='green')
 
 
 @schedule.command(
     help="List all runs of the workflow on Aero scheduler.")
 @click.option("--flow-id", default=None,
-    help="List all runs of the workflow in ABORTED state on "
-        "the Aero scheduler.")
+              help="List all runs of the workflow in ABORTED state on "
+              "the Aero scheduler.")
 @click.pass_obj
 def get_logs(obj, flow_id=None):
 
-    from ..aws_client import get_aws_client 
+    from ..aws_client import get_aws_client
 
     # Get AWS Clients
     sfn = get_aws_client('stepfunctions')
@@ -434,7 +472,7 @@ def get_logs(obj, flow_id=None):
     # Execution (Run) > Step > Task
     question_arr = [
         ask_execution_question,
-        ask_step_question, 
+        ask_step_question,
         ask_task_question
     ]
 
@@ -449,7 +487,7 @@ def get_logs(obj, flow_id=None):
             cloudwatch=cloudwatch
         )
 
-        # If back or 
+        # If back or
         if resp == BACK_KEYWORD:
             question -= 1
         # If we are at the end, stay on the last question
@@ -457,6 +495,15 @@ def get_logs(obj, flow_id=None):
             question += 1
 
 
+def find_parameter(lst, param_to_find, parameter):
+
+    for x in lst:
+        if x[parameter] == param_to_find:
+            return x
+
+    return None
+
+
 def get_key(param, key):
     try:
         lst = param['Container']['Environment']
@@ -477,9 +524,9 @@ def build_event_tree(execution_history):
     # Events are in order
     for task in execution_history['events']:
 
-        # If task has scheduled 
+        # If task has scheduled
         if task['type'] == "TaskScheduled":
-        
+
             # Some intermediate tasks arent actually steps - so skip
             if task['taskScheduledEventDetails'].get('resourceType', "") == "dynamodb":
                 continue
@@ -505,7 +552,7 @@ def build_event_tree(execution_history):
 
             # Tasks can be attempted multiple times, so iterate through these
             for attempt in output['Attempts']:
-                
+
                 # If split index in the step dict already
                 if split_index in tree[step_name]:
                     # Add task state and logid
@@ -513,7 +560,7 @@ def build_event_tree(execution_history):
                 else:
                     # Add task state and logid
                     tree[step_name][split_index] = [("TaskSucceeded", attempt['Container']['LogStreamName'])]
-                    
+
         if task['type'] == "TaskFailed":
 
             if task['taskFailedEventDetails'].get('resourceType', "") == "dynamodb":
@@ -546,15 +593,15 @@ def build_event_tree(execution_history):
             ...
         }
     '''
-                    
+
     return tree
-   
+
 
 def ask_task_question(obj, cloudwatch=None, sfn=None):
 
     # If there is only one task (likely for most steps), open the task logs
     if len(obj['tree'][obj['step']].keys()) == 1:
-        
+
         # fetch logs for tree[step][0][0][1]
         open_log(obj['tree'][obj['step']][0][0][1], cloudwatch)
         # If we dont BACK, then this function keeps being run, then opening logs
@@ -592,7 +639,7 @@ def ask_step_question(obj, cloudwatch=None, sfn=None):
             for attempts in tree[key][index]:
                 if attempts[0] == "TaskFailed":
                     has_failed = True
-        
+
         steps.append(f"FAILED: {key}" if has_failed else key)
 
     step = ask_questions(
@@ -653,20 +700,20 @@ def check_if_back_or_exit(answer):
     if answer == EXIT_KEYWORD:
         import sys
         sys.exit(0)
-    
+
 def chunked(lst, n):
     return [lst[i:i + n] for i in range(0, len(lst), n)]
 
 def generate_control_opts(i, l, enable_back=False):
-    lst = [BACK_KEYWORD] if enable_back else [] 
+    lst = [BACK_KEYWORD] if enable_back else []
 
     if i != 0:
         lst.append("Previous Page")
     if i != (l-1):
         lst.append("Next Page")
-    
+
     lst.append(EXIT_KEYWORD)
-    
+
     return lst
 
 def ask_questions(question, choices, enable_back=False):
@@ -678,20 +725,20 @@ def ask_questions(question, choices, enable_back=False):
     while True:
         questions = [
             inquirer.List("var",
-                        message=question,
-                        choices=chunked_choices[i] + \
+                          message=question,
+                          choices=chunked_choices[i] + \
                             generate_control_opts(i, len(chunked_choices), enable_back),
-                    ),
+                          ),
         ]
-        response =  inquirer.prompt(questions)["var"]
+        response = inquirer.prompt(questions)["var"]
 
         if response == "Next Page":
-            i +=1
+            i += 1
         elif response == "Previous Page":
-            i -=1
+            i -= 1
         else:
             return response
-        
+
 def open_log(log_stream, cloudwatch):
 
     resp = cloudwatch.get_log_events(
@@ -711,7 +758,7 @@ def open_log(log_stream, cloudwatch):
             nextToken=token
         )
         logs.extend(resp['events'])
-    
+
     str_logs = ""
 
     for log in logs:
@@ -724,12 +771,8 @@ def open_log(log_stream, cloudwatch):
     try:
         with os.fdopen(fd, 'w') as tmp:
             tmp.write(str_logs)
-        
+
         cmd = os.environ.get('EDITOR', 'vi') + ' ' + path
         subprocess.call(cmd, shell=True)
     finally:
         os.remove(path)
-
-
-
-
