@@ -22,7 +22,6 @@ def bootstrap_environment(flow_name, env_id):
 
 
 def setup_conda_manifest(flow_name):
-    print(DATASTORE_LOCAL_DIR)
     manifest_folder = os.path.join(os.getcwd(), DATASTORE_LOCAL_DIR, flow_name)
     if not os.path.exists(manifest_folder):
         os.makedirs(manifest_folder)
@@ -52,10 +51,14 @@ def install_conda_environment(env_id, packages):
         "if ! type conda  >/dev/null 2>&1; \
             then wget --no-check-certificate https://repo.anaconda.com/pkgs/misc/conda-execs/conda-latest-linux-64.exe -O conda >/dev/null 2>&1; \
             chmod +x conda; \
-            export PATH=$PATH:{0}; fi".format(os.getcwd()),
-        'cd {0}'.format(os.path.join(os.getcwd(), 'pkgs')),
-        'conda create --yes --no-default-packages -p {0} --no-deps {1}'.format(os.path.join(os.getcwd(), env_id), ' '.join(packages)),
-        'cd {0}'.format(os.getcwd())
+            export PATH=$PATH:{0}; fi".format(
+            os.getcwd()
+        ),
+        "cd {0}".format(os.path.join(os.getcwd(), "pkgs")),
+        "conda create --yes --no-default-packages -p {0} --no-deps {1} >/dev/null 2>&1".format(
+            os.path.join(os.getcwd(), env_id), " ".join(packages)
+        ),
+        "cd {0}".format(os.getcwd()),
     ]
     if ENV_ESCAPE_PY is not None:
         cwd = os.getcwd()
@@ -68,4 +71,4 @@ def install_conda_environment(env_id, packages):
 
 
 if __name__ == "__main__":
-    bootstrap_environment(sys.argv[1], sys.argv[2])
+    bootstrap_environment(sys.argv[1], sys.argv[2])
\ No newline at end of file
