@@ -23,18 +23,22 @@ try:
 except:
     blockingError = OSError
 
+
 class PipeUnavailableError(Exception):
     """raised when unable to write to pipe given allotted time"""
+
     pass
 
 
 class NullSidecarError(Exception):
-    """raised when tyring to poll or interact with the fake subprocess in the null sidecar"""
+    """raised when trying to poll or interact with the fake subprocess in the null sidecar"""
+
     pass
 
 
 class MsgTimeoutError(Exception):
-    """raised when tyring unable to send message to sidecar in allocated time"""
+    """raised when trying unable to send message to sidecar in allocated time"""
+
     pass
 
 
@@ -44,7 +48,6 @@ class NullPoller(object):
 
 
 class SidecarSubProcess(object):
-
     def __init__(self, worker_type):
         # type: (str) -> None
         self.__worker_type = worker_type
@@ -54,20 +57,28 @@ class SidecarSubProcess(object):
 
     def start(self):
 
-        if (self.__worker_type is not None and \
-                self.__worker_type.startswith(NULL_SIDECAR_PREFIX)) or \
-                (platform.system() == 'Darwin' and sys.version_info < (3, 0)):
-                # if on darwin and running python 2 disable sidecars
-                # there is a bug with importing poll from select in some cases
-                self.__poller = NullPoller()
+        if (
+            self.__worker_type is not None
+            and self.__worker_type.startswith(NULL_SIDECAR_PREFIX)
+        ) or (platform.system() == "Darwin" and sys.version_info < (3, 0)):
+            # if on darwin and running python 2 disable sidecars
+            # there is a bug with importing poll from select in some cases
+            #
+            # TODO: Python 2 shipped by Anaconda allows for
+            # `from select import poll`. We can consider enabling sidecars
+            # for that distribution if needed at a later date.
+            self.__poller = NullPoller()
 
         else:
             from select import poll
+
             python_version = sys.executable
-            cmdline = [python_version,
-                       '-u',
-                       os.path.dirname(__file__) + '/sidecar_worker.py',
-                       self.__worker_type]
+            cmdline = [
+                python_version,
+                "-u",
+                os.path.dirname(__file__) + "/sidecar_worker.py",
+                self.__worker_type,
+            ]
             debug.sidecar_exec(cmdline)
 
             self.__process = self.__start_subprocess(cmdline)
@@ -75,21 +86,25 @@ class SidecarSubProcess(object):
             if self.__process is not None:
                 fcntl.fcntl(self.__process.stdin, F_SETFL, O_NONBLOCK)
                 self.__poller = poll()
-                self.__poller.register(self.__process.stdin.fileno(),
-                                       select.POLLOUT)
+                self.__poller.register(self.__process.stdin.fileno(), select.POLLOUT)
             else:
                 # unable to start subprocess, fallback to Null sidecar
-                self.logger("unable to start subprocess for sidecar %s"
-                      % self.__worker_type)
+                self.logger(
+                    "unable to start subprocess for sidecar %s" % self.__worker_type
+                )
                 self.__poller = NullPoller()
 
     def __start_subprocess(self, cmdline):
         for i in range(3):
             try:
-                return subprocess.Popen(cmdline,
-                                        stdin=subprocess.PIPE,
-                                        stdout=open(os.devnull, 'w'),
-                                        bufsize=0)
+                # Set stdout=sys.stdout & stderr=sys.stderr
+                # to print to console the output of sidecars.
+                return subprocess.Popen(
+                    cmdline,
+                    stdin=subprocess.PIPE,
+                    stdout=open(os.devnull, "w"),
+                    bufsize=0,
+                )
             except blockingError as be:
                 self.logger("warning: sidecar popen failed: %s" % repr(be))
             except Exception as e:
@@ -104,7 +119,7 @@ class SidecarSubProcess(object):
             pass
 
     def emit_msg(self, msg):
-        msg_ser = msg.serialize().encode('utf-8')
+        msg_ser = msg.serialize().encode("utf-8")
         written_bytes = 0
         while written_bytes < len(msg_ser):
             try:
@@ -113,7 +128,7 @@ class SidecarSubProcess(object):
                     raise MsgTimeoutError("poller timed out")
                 for fd, event in fds:
                     if event & select.POLLERR:
-                        raise PipeUnavailableError('pipe unavailable')
+                        raise PipeUnavailableError("pipe unavailable")
                     f = os.write(fd, msg_ser[written_bytes:])
                     written_bytes += f
             except NullSidecarError:
@@ -132,10 +147,10 @@ class SidecarSubProcess(object):
                 self.start()
             if retries > 0:
                 self.logger("retrying msg send to sidecar")
-                self.msg_handler(msg, retries-1)
+                self.msg_handler(msg, retries - 1)
             else:
                 self.logger("error sending log message")
                 self.logger(repr(ex))
 
     def logger(self, msg):
-        print("metaflow logger: " + msg, file=sys.stderr)
+        print("metaflow sidecar logger: " + msg, file=sys.stderr)
