@@ -17,6 +17,7 @@ from metaflow.plugins import SIDECARS
 
 class WorkershutdownError(Exception):
     """raised when terminating sidecar"""
+
     pass
 
 
@@ -44,7 +45,7 @@ def process_messages(worker):
 
 
 @click.command(help="Initialize workers")
-@click.argument('worker-type')
+@click.argument("worker-type")
 def main(worker_type):
 
     worker_process = SIDECARS.get(worker_type)
